/*------------------------------------------------------------------------ *
 * \file	evmomapl137_aic3106_init.c                                     *
 * \brief	implementation of a aic3106 driver for the OMAP-L137 EVM       *
 * ----------------------------------------------------------------------- */

//  ======== Includes =======================================================
#include "evmomapl137_aic3106.h"
#include "evmomapl137_i2c.h"

//  ======== Definitions ====================================================
#define AIC3106_I2C_ADDR          0x18    // I2C address

//  ======== EVMOMAPL137_AIC3106_rset =======================================
// write a register on the AIC3106
void EVMOMAPL137_AIC3106_rset( Uint16 regnum, Uint16 regval )
{
    Uint8 cmd[2];
    cmd[0] = regnum & 0x007F;       // 7-bit Device Address
    cmd[1] = regval;                // 8-bit Register Data

    EVMOMAPL137_I2C_write( AIC3106_I2C_ADDR, cmd, 2 );

    return;
}

/* ------------------------------------------------------------------------ *
 *  Initialize AIC3106 codec by writing to its control registers            *
 *																		    *
 *      AIC3106.MCLK = PLL1705.SCK02                                        *
 *                      FS = ( AIC3106.MCLK * K ) / ( 2048 * P )            *
 *                                                                          *
 *                       or a FS=[48 kHz] & MCLK=[22.5792 MHz]              *
 *                          : 48kHz = ( 22.5792 MHz * K ) / ( 2048 * P )    *
 *                          : P = 2, K[J.D] = 8.7075                        *
 * ------------------------------------------------------------------------ */

//  ======== EVMOMAPL137_AIC3106_init =======================================
//  configuration codec registers. Please see AIC3106 documentation for explanation (TLV320AIC3106)
void EVMOMAPL137_AIC3106_init( Uint16 adc_gain, Uint16 dac_atten, Uint16 fsAIC3106 )
{
    /* configure AIC3106 */
    EVMOMAPL137_AIC3106_rset( AIC3106_PAGESELECT, 0 );       // select page 0
    EVMOMAPL137_AIC3106_rset( AIC3106_RESET, 0x80 );         // reset AIC31

    /* configure AIC3106 register 2 and 7 for setup sampling rate */
    switch( fsAIC3106 ) // default 8kHz
    {
    	case FS_8000_HZ:
    	  EVMOMAPL137_AIC3106_rset(  2, 0xAA );	 // ADC/DAC sample rate = f(s)/6 = MCLK/(128*Q)/6
          EVMOMAPL137_AIC3106_rset(  7, 0x0A );  // codec Datapath Setup [LeftDAC=LEFT][RightDAC=RIGHT]
    	  break;
    	case FS_12000_HZ:
    	  EVMOMAPL137_AIC3106_rset(  2, 0x66 );	 // ADC/DAC sample rate = f(s)/4 = MCLK/(128*Q)/4
          EVMOMAPL137_AIC3106_rset(  7, 0x0A );  // codec Datapath Setup [LeftDAC=LEFT][RightDAC=RIGHT]
    	  break;
    	case FS_16000_HZ:
    	  EVMOMAPL137_AIC3106_rset(  2, 0x44 );	 // ADC/DAC sample rate = f(s)/3 = MCLK/(128*Q)/3
          EVMOMAPL137_AIC3106_rset(  7, 0x0A );  // codec Datapath Setup [LeftDAC=LEFT][RightDAC=RIGHT]
    	  break;
    	case FS_24000_HZ:
    	  EVMOMAPL137_AIC3106_rset(  2, 0x22 );	 // ADC/DAC sample rate = f(s)/2 = MCLK/(128*Q)/2
          EVMOMAPL137_AIC3106_rset(  7, 0x0A );  // codec Datapath Setup [LeftDAC=LEFT][RightDAC=RIGHT]
    	  break;
    	case FS_48000_HZ:
    	  EVMOMAPL137_AIC3106_rset(  2, 0x00 );	 // ADC/DAC sample rate = f(s)/1 = MCLK/(128*Q)/1
          EVMOMAPL137_AIC3106_rset(  7, 0x0A );  // codec Datapath Setup [LeftDAC=LEFT][RightDAC=RIGHT]
          break;
    	default:
    	EVMOMAPL137_AIC3106_rset(  2, 0xAA );	 // ADC/DAC sample rate = f(s)/6 = MCLK/(128*Q)/6
    	EVMOMAPL137_AIC3106_rset(  7, 0x0A );    // codec Datapath Setup [LeftDAC=LEFT][RightDAC=RIGHT]
    	  break;
    }

    /* configure AIC3106 registers */
    EVMOMAPL137_AIC3106_rset( 3, 0x22 );         // 3 PLL A                            <- [PLL=OFF][Q=4][P=2]
    EVMOMAPL137_AIC3106_rset( 4, 0x20 );         // 4 PLL B                            <- [J=8]
    EVMOMAPL137_AIC3106_rset( 5, 0x6E );         // 5 PLL C                            <- [D=7075]
    EVMOMAPL137_AIC3106_rset( 6, 0x23 );         // 6 PLL D                            <- [D=7075]
    EVMOMAPL137_AIC3106_rset( 8, 0x00 );         // 8  Audio InterFace Control A       <- [BCLK=Slave][MCLK=Slave]
    EVMOMAPL137_AIC3106_rset( 9, 0x00 );         // 9  Audio InterFace Control B       <- [I2S mode][16 bit]
    EVMOMAPL137_AIC3106_rset( 10, 0x00 );        // 10 Audio InterFace Control C       <- [Data oFFset=0]
    EVMOMAPL137_AIC3106_rset( 12, 0x00 );        // 12 Audio InterFace Control C       <- [Data oFFset=0]
    EVMOMAPL137_AIC3106_rset( 15, adc_gain);     // 15  left ADC PGA Gain              <- [Mute=OFF][Gain=0dB]
    EVMOMAPL137_AIC3106_rset( 16, adc_gain);     // 16 Right ADC PGA Gain              <- [Mute=OFF][Gain=0dB]
    EVMOMAPL137_AIC3106_rset( 17, 0x0F);         // 17 MIC3L/R to  left ADC            <- [MIC3L=0dBGain][MIC3R=NotConnect]
    EVMOMAPL137_AIC3106_rset( 18, 0xF0 );        // 18 MIC3L/R to Right ADC            <- [MIC3L=NotConnect][MIC3R=0dBGain]
    EVMOMAPL137_AIC3106_rset( 19 ,0x04 );        // 19  LINE1L to  left ADC            <- [SingleEnd][Gain=0dB][Power=ON][SoftStep=OncePerFS]
    EVMOMAPL137_AIC3106_rset( 22, 0x04 );        // 22  LINE1R to Right ADC            <- [SingleEnd][Gain=0dB][Power=ON][SoftStep=OncePerFS]
    EVMOMAPL137_AIC3106_rset( 25, 0x40 );        // 25 MICBIAS                         <- [MICBIAS=2.0V]
    EVMOMAPL137_AIC3106_rset( 27,0 );            // 27  left AGC B                     <- [OFF]
    EVMOMAPL137_AIC3106_rset( 30, 0 );           // 30 Right AGC B                     <- [OFF]
    EVMOMAPL137_AIC3106_rset( 37, 0xE0 );        // 37 DAC Power & Output Dvr          <- [leftDAC=ON][RightDAC=ON][HPLCOM=SingleEnd]
    EVMOMAPL137_AIC3106_rset( 38, 0x10 );        // 38 High Power Output Dvr           <- [HPRCOM=SingleEnd][ShortCircuit=OFF]
    EVMOMAPL137_AIC3106_rset( 43, dac_atten );   // 43  left DAC Digital Volume        <- [Mute=OFF][Gain=0dB]
    EVMOMAPL137_AIC3106_rset( 44, dac_atten );   // 44 Right DAC Digital Volume        <- [Mute=OFF][Gain=0dB]
    EVMOMAPL137_AIC3106_rset( 47, 0x80 );        // 47 DAC_L1 to HPLOUT Volume         <- [Routed]
    EVMOMAPL137_AIC3106_rset( 51, 0x09 );        // 51           HPLOUT Output         <- [Mute=OFF][Power=ON]
    EVMOMAPL137_AIC3106_rset( 58, 0 );           // 58           HPLCOM Output         <- []
    EVMOMAPL137_AIC3106_rset( 64, 0x80 );        // 64 DAC_R1 to HPROUT Volume         <- [Routed]
    EVMOMAPL137_AIC3106_rset( 65, 0x09 );        // 65           HPROUT Output         <- [Mute=OFF][Power=ON]
    EVMOMAPL137_AIC3106_rset( 72, 0 );           // 72           HPRCOM Output         <- []
    EVMOMAPL137_AIC3106_rset( 82, 0x80 );        // 82 DAC_L1 to left_LOP/M Volume     <- [Routed]
    EVMOMAPL137_AIC3106_rset( 86 ,0x09 );        // 83 LINE2R to left_LOP/M Volume     <- []
    EVMOMAPL137_AIC3106_rset( 92, 0x80 );        // 92 DAC_R1 to RIGHT_LOP/M Volume    <- [Routed]
    EVMOMAPL137_AIC3106_rset( 93, 0x09 );        // 93           RIGHT_LOP/M Output    <- [Mute=OFF][Power=ON]
    EVMOMAPL137_AIC3106_rset( 101, 0x01 );       // 101 GPIO Control Register B        <- [CODEC_CLKIN = CLKDIV_OUT]
    EVMOMAPL137_AIC3106_rset( 102, 0 );          // 102 Clock Generation Control       <- [PLLCLK_IN and CLKDIV_IN use MCLK]

    return;
}


