/*------------------------------------------------------------------------ *
 * \file	evmomapl137_i2c.c                                              *
 * \brief	implementation of a I2C driver for the OMAP-L137 EVM           *
 * ----------------------------------------------------------------------- */

/* ------------------------------------------------------------------------ *
 *                                                                          *
 *      I2C in Master mode                                                  *
 *                                                                          *
 *      i2c_addr    <- I2C slave address                                    *
 *      data        <- I2C data ptr                                         *
 *      len         <- # of bytes to write                                  *
 *                                                                          *
 * ------------------------------------------------------------------------ */

//  ======== Includes =======================================================
#include "evmomapl137_i2c.h"

//  ========  Private Defines and Macros ====================================
Int32 i2c_timeout = 0x10000;

//  ======== EVMOMAPL137_I2C_init( )  =======================================
//  enable and initialize the I2C module
Int16 EVMOMAPL137_I2C_init( )
{
    I2C_ICMDR   = 0;                // reset I2C
    I2C_ICPSC   = 23;               // prescale to get 1MHz I2C internal
    I2C_ICCLKL  = 20;               // configuration clk LOW for 20kHz
    I2C_ICCLKH  = 20;               // configuration clk HIGH for 20kHz
    I2C_ICMDR  |= ICMDR_IRS;        // release I2C from reset
    return 0;
}

//  ======== EVMOMAPL137_I2C_Close( )  =======================================
Int16 EVMOMAPL137_I2C_close( )
{
        I2C_ICMDR = 0;                      // reset I2C
        return 0;
}

//  ======== EVMOMAPL137_I2C_reset( )  =======================================
Int16 EVMOMAPL137_I2C_reset( )
{
    EVMOMAPL137_I2C_close( );
    EVMOMAPL137_I2C_init( );
    return 0;
}

//  ======== EVMOMAPL137_I2C_write( )  =======================================
Int16 EVMOMAPL137_I2C_write( Uint16 i2c_addr, Uint8* data, Uint16 len )
{
    Int32 timeout, i;


        I2C_ICCNT = len;                    // set length
        I2C_ICSAR = i2c_addr;               // set I2C slave address
        I2C_ICMDR = ICMDR_STT               // set for Master Write
                  | ICMDR_TRX
                  | ICMDR_MST
                  | ICMDR_IRS
                  | ICMDR_FREE;

        EVMOMAPL137_wait( 10 );             // short delay

        for ( i = 0 ; i < len ; i++ )
        {
            I2C_ICDXR = data[i];            // write

            timeout = i2c_timeout;
            do
            {
                if ( timeout-- < 0  )
                {
                    EVMOMAPL137_I2C_reset( );
                    return -1;
                }
            } while ( ( I2C_ICSTR & ICSTR_ICXRDY ) == 0 );   // wait for Tx Ready
        }

        I2C_ICMDR |= ICMDR_STP;             // generate STOP

        return 0;

}

//  ======== EVMOMAPL137_I2C_read( )  =========================================
Int16 EVMOMAPL137_I2C_read( Uint16 i2c_addr, Uint8* data, Uint16 len )
{
    Int32 timeout, i;

    I2C_ICCNT = len;                    // set length
    I2C_ICSAR = i2c_addr;               // set I2C slave address
    I2C_ICMDR = ICMDR_STT               // set for Master Read
              | ICMDR_MST
              | ICMDR_IRS
              | ICMDR_FREE;

    EVMOMAPL137_wait( 10 );             // short delay

    for ( i = 0 ; i < len ; i++ )
    {
        timeout = i2c_timeout;

        /* wait for Rx Ready */
        do
        {
            if ( timeout-- < 0 )
            {
                EVMOMAPL137_I2C_reset( );
                return -1;
            }
        } while ( ( I2C_ICSTR & ICSTR_ICRRDY ) == 0 );// wait for Rx Ready

        data[i] = I2C_ICDRR;            // read
    }

      //I2C_ICMDR |= ICMDR_STP;             // generate STOP

        return 0;
}

//  ======== EVMOMAPL137_I2C_read_variable( )  =================================
Int16 EVMOMAPL137_I2C_read_variable( Uint16 i2c_addr, Uint8* data, Uint16 len )
{
    Int32 timeout, i;


    I2C_ICCNT = len;                    // set length
    I2C_ICSAR = i2c_addr;               // set I2C slave address
    I2C_ICMDR = ICMDR_STT               // set for Master Read
              | ICMDR_MST
              | ICMDR_IRS
              | ICMDR_FREE;

    EVMOMAPL137_wait( 10 );                        // short delay

    for ( i = 0 ; i < len ; i++ )
    {
        if ( i == 1 )
        {
            len = data[0];
            I2C_ICCNT = len;            // set length
        }

        timeout = i2c_timeout;
        do
        {
            if ( timeout-- < 0 )
            {
                EVMOMAPL137_I2C_reset( );
                return -1;
            }
        } while ( ( I2C_ICSTR & ICSTR_ICRRDY ) == 0 );// wait for Rx Ready

        data[i] = I2C_ICDRR;            // read
    }

    I2C_ICMDR |= ICMDR_STP;             // generate STOP

    return 0;
}
